@@ -26,7 +26,7 @@ import sys
 from decimal import Context, Decimal, localcontext
 from fractions import Fraction
 from functools import reduce
-from inspect import isclass
+from inspect import isabstract, isclass
 from uuid import UUID
 
 import attr
@@ -38,7 +38,6 @@ from hypothesis.internal.cache import LRUReusedCache
 from hypothesis.internal.cathetus import cathetus
 from hypothesis.internal.charmap import as_general_categories
 from hypothesis.internal.compat import (
-    abc,
     ceil,
     floor,
     gcd,
@@ -70,6 +69,7 @@ from hypothesis.internal.reflection import (
     nicerepr,
     proxies,
     required_args,
+    reserved_means_kwonly_star,
 )
 from hypothesis.internal.validation import (
     check_type,
@@ -83,10 +83,12 @@ from hypothesis.internal.validation import (
 )
 from hypothesis.searchstrategy import SearchStrategy, check_strategy
 from hypothesis.searchstrategy.collections import (
+    FixedAndOptionalKeysDictStrategy,
     FixedKeysDictStrategy,
     ListStrategy,
     TupleStrategy,
     UniqueListStrategy,
+    UniqueSampledListStrategy,
 )
 from hypothesis.searchstrategy.datetime import (
     DateStrategy,
@@ -96,11 +98,7 @@ from hypothesis.searchstrategy.datetime import (
 from hypothesis.searchstrategy.deferred import DeferredStrategy
 from hypothesis.searchstrategy.functions import FunctionStrategy
 from hypothesis.searchstrategy.lazy import LazyStrategy
-from hypothesis.searchstrategy.misc import (
-    BoolStrategy,
-    JustStrategy,
-    SampledFromStrategy,
-)
+from hypothesis.searchstrategy.misc import JustStrategy, SampledFromStrategy
 from hypothesis.searchstrategy.numbers import (
     BoundedIntStrategy,
     FixedBoundedFloatStrategy,
@@ -128,16 +126,11 @@ try:
 except ImportError:
     pass
 
-try:
-    import numpy
-except ImportError:
-    numpy = None
-
 if False:
     import random  # noqa
     from types import ModuleType  # noqa
     from typing import Any, Dict, Union, Sequence, Callable, Pattern  # noqa
-    from typing import TypeVar, Tuple, List, Set, FrozenSet, overload  # noqa
+    from typing import TypeVar, Tuple, Iterable, List, Set, FrozenSet, overload  # noqa
     from typing import Type, Text, AnyStr, Optional  # noqa
 
     from hypothesis.utils.conventions import InferType  # noqa
@@ -182,18 +175,17 @@ def cacheable(fn):
     # type: (T) -> T
     @proxies(fn)
     def cached_strategy(*args, **kwargs):
-        kwargs_cache_key = set()
         try:
-            for k, v in kwargs.items():
-                kwargs_cache_key.add((k, convert_value(v)))
+            kwargs_cache_key = {(k, convert_value(v)) for k, v in kwargs.items()}
         except TypeError:
             return fn(*args, **kwargs)
         cache_key = (fn, tuple(map(convert_value, args)), frozenset(kwargs_cache_key))
         try:
-            return STRATEGY_CACHE[cache_key]
+            if cache_key in STRATEGY_CACHE:
+                return STRATEGY_CACHE[cache_key]
         except TypeError:
             return fn(*args, **kwargs)
-        except KeyError:
+        else:
             result = fn(*args, **kwargs)
             if not isinstance(result, SearchStrategy) or result.is_cacheable:
                 STRATEGY_CACHE[cache_key] = result
@@ -242,7 +234,7 @@ class Nothing(SearchStrategy):
     def do_draw(self, data):
         # This method should never be called because draw() will mark the
         # data as invalid immediately because is_empty is True.
-        assert False  # pragma: no cover
+        raise NotImplementedError("This should never happen")  # pragma: no cover
 
     def calc_has_reusable_values(self, recur):
         return True
@@ -346,7 +338,7 @@ def one_of(*args):
 @cacheable
 @defines_strategy_with_reusable_values
 def integers(min_value=None, max_value=None):
-    # type: (Real, Real) -> SearchStrategy[int]
+    # type: (int, int) -> SearchStrategy[int]
     """Returns a strategy which generates integers; in Python 2 these may be
     ints or longs.
 
@@ -426,7 +418,7 @@ def booleans():
     Examples from this strategy will shrink towards False (i.e.
     shrinking will try to replace True with False where possible).
     """
-    return BoolStrategy()
+    return sampled_from([False, True])
 
 
 @cacheable
@@ -488,7 +480,7 @@ def floats(
             "Got width=%r, but the only valid values are the integers 16, "
             "32, and 64." % (width,)
         )
-    if width == 16 and sys.version_info[:2] < (3, 6) and numpy is None:
+    if width == 16 and sys.version_info[:2] < (3, 6) and "numpy" not in sys.modules:
         raise InvalidArgument(  # pragma: no cover
             "width=16 requires either Numpy, or Python >= 3.6"
         )
@@ -576,26 +568,31 @@ def floats(
     elif max_value == float("-inf"):
         raise InvalidArgument("allow_infinity=False excludes max_value=-inf")
 
+    unbounded_floats = FloatStrategy(
+        allow_infinity=allow_infinity, allow_nan=allow_nan, width=width
+    )
+
     if min_value is None and max_value is None:
-        result = FloatStrategy(
-            allow_infinity=allow_infinity, allow_nan=allow_nan
-        )  # type: SearchStrategy[float]
+        return unbounded_floats
     elif min_value is not None and max_value is not None:
         if min_value == max_value:
             assert isinstance(min_value, float)
             result = just(min_value)
         elif is_negative(min_value):
             if is_negative(max_value):
-                result = floats(min_value=-max_value, max_value=-min_value).map(
-                    operator.neg
-                )
-            else:
-                result = floats(min_value=0.0, max_value=max_value) | floats(
-                    min_value=0.0, max_value=-min_value
+                return floats(
+                    min_value=-max_value, max_value=-min_value, width=width
                 ).map(operator.neg)
+            else:
+                return one_of(
+                    floats(min_value=0.0, max_value=max_value, width=width),
+                    floats(min_value=0.0, max_value=-min_value, width=width).map(
+                        operator.neg
+                    ),
+                )
         elif count_between_floats(min_value, max_value) > 1000:
-            result = FixedBoundedFloatStrategy(
-                lower_bound=min_value, upper_bound=max_value
+            return FixedBoundedFloatStrategy(
+                lower_bound=min_value, upper_bound=max_value, width=width
             )
         else:
             ub_int = float_to_int(max_value, width)
@@ -606,42 +603,34 @@ def floats(
             )
     elif min_value is not None:
         assert isinstance(min_value, float)
-        if min_value < 0:
-            result = floats(min_value=0.0, allow_infinity=allow_infinity) | floats(
-                min_value=min_value, max_value=-0.0
+        if is_negative(min_value):
+            return one_of(
+                unbounded_floats.map(abs),
+                floats(min_value=min_value, max_value=-0.0, width=width),
             )
         else:
-            result = floats(allow_infinity=allow_infinity, allow_nan=False).map(
-                lambda x: min_value + abs(x)  # type: ignore
-            )
-            if not allow_infinity:
-                result = result.filter(lambda x: not math.isinf(x))
-        if min_value == 0 and not is_negative(min_value):
-            result = result.filter(lambda x: math.copysign(1.0, x) == 1)
+            result = unbounded_floats.map(lambda x: min_value + abs(x))
     else:
         assert isinstance(max_value, float)
-        if max_value > 0:
-            result = floats(min_value=0.0, max_value=max_value) | floats(
-                max_value=-0.0, allow_infinity=allow_infinity
+        if not is_negative(max_value):
+            return one_of(
+                floats(min_value=0.0, max_value=max_value, width=width),
+                unbounded_floats.map(lambda x: -abs(x)),
             )
         else:
-            result = floats(allow_infinity=allow_infinity, allow_nan=False).map(
-                lambda x: max_value - abs(x)  # type: ignore
-            )
-            if not allow_infinity:
-                result = result.filter(lambda x: not math.isinf(x))
-        if max_value == 0 and is_negative(max_value):
-            result = result.filter(is_negative)
+            result = unbounded_floats.map(lambda x: max_value - abs(x))
 
     if width < 64:
 
         def downcast(x):
             try:
                 return float_of(x, width)
-            except OverflowError:
+            except OverflowError:  # pragma: no cover
                 reject()
 
-        return result.map(downcast)
+        result = result.map(downcast)
+    if not allow_infinity:
+        result = result.filter(lambda x: not math.isinf(x))
     return result
 
 
@@ -724,14 +713,13 @@ def lists(
     # type: (...) -> SearchStrategy[List[Ex]]
     """Returns a list containing values drawn from elements with length in the
     interval [min_size, max_size] (no bounds in that direction if these are
-    None). If max_size is 0 then elements may be None and only the empty list
-    will be drawn.
+    None). If max_size is 0, only the empty list will be drawn.
 
-    If unique is True (or something that evaluates to True), we compare direct
+    If ``unique`` is True (or something that evaluates to True), we compare direct
     object equality, as if unique_by was ``lambda x: x``. This comparison only
     works for hashable types.
 
-    If unique_by is not None it must be a callable or tuple of callables
+    If ``unique_by`` is not None it must be a callable or tuple of callables
     returning a hashable type when given a value drawn from elements. The
     resulting list will satisfy the condition that for ``i`` != ``j``,
     ``unique_by(result[i])`` != ``unique_by(result[j])``.
@@ -776,6 +764,26 @@ def lists(
         for i, f in enumerate(unique_by):
             if not callable(f):
                 raise InvalidArgument("unique_by[%i]=%r is not a callable" % (i, f))
+        # Note that lazy strategies automatically unwrap when passed to a defines_strategy
+        # function.
+        if isinstance(elements, SampledFromStrategy):
+            element_count = len(elements.elements)
+            if min_size > element_count:
+                raise InvalidArgument(
+                    "Cannot create a collection of min_size=%r unique elements with "
+                    "values drawn from only %d distinct elements"
+                    % (min_size, element_count)
+                )
+
+            if max_size is not None:
+                max_size = min(max_size, element_count)
+            else:
+                max_size = element_count
+
+            return UniqueSampledListStrategy(
+                elements=elements, max_size=max_size, min_size=min_size, keys=unique_by
+            )
+
         return UniqueListStrategy(
             elements=elements, max_size=max_size, min_size=min_size, keys=unique_by
         )
@@ -836,7 +844,14 @@ class PrettyIter(object):
 
 
 @defines_strategy
-def iterables(elements, min_size=0, max_size=None, unique_by=None, unique=False):
+def iterables(
+    elements,  # type: SearchStrategy[Ex]
+    min_size=0,  # type: int
+    max_size=None,  # type: int
+    unique_by=None,  # type: Union[Callable, Tuple[Callable, ...]]
+    unique=False,  # type: bool
+):
+    # type: (...) -> SearchStrategy[Iterable[Ex]]
     """This has the same behaviour as lists, but returns iterables instead.
 
     Some iterables cannot be indexed (e.g. sets) and some do not have a
@@ -854,8 +869,13 @@ def iterables(elements, min_size=0, max_size=None, unique_by=None, unique=False)
 
 
 @defines_strategy
-def fixed_dictionaries(mapping):
-    # type: (Dict[T, SearchStrategy[Ex]]) -> SearchStrategy[Dict[T, Ex]]
+@reserved_means_kwonly_star
+def fixed_dictionaries(
+    mapping,  # type: Dict[T, SearchStrategy[Ex]]
+    __reserved=not_set,  # type: Any
+    optional=None,  # type: Dict[T, SearchStrategy[Ex]]
+):
+    # type: (...) -> SearchStrategy[Dict[T, Ex]]
     """Generates a dictionary of the same type as mapping with a fixed set of
     keys mapping to strategies. mapping must be a dict subclass.
 
@@ -868,8 +888,25 @@ def fixed_dictionaries(mapping):
     the generated dictionary.
     """
     check_type(dict, mapping, "mapping")
-    for v in mapping.values():
-        check_strategy(v)
+    for k, v in mapping.items():
+        check_strategy(v, "mapping[%r]" % (k,))
+    if __reserved is not not_set:
+        raise InvalidArgument("Do not pass __reserved; got %r" % (__reserved,))
+    if optional is not None:
+        check_type(dict, optional, "optional")
+        for k, v in optional.items():
+            check_strategy(v, "optional[%r]" % (k,))
+        if type(mapping) != type(optional):
+            raise InvalidArgument(
+                "Got arguments of different types: mapping=%s, optional=%s"
+                % (nicerepr(type(mapping)), nicerepr(type(optional)))
+            )
+        if set(mapping) & set(optional):
+            raise InvalidArgument(
+                "The following keys were in both mapping and optional, "
+                "which is invalid: %r" % (set(mapping) & set(optional))
+            )
+        return FixedAndOptionalKeysDictStrategy(mapping, optional)
     return FixedKeysDictStrategy(mapping)
 
 
@@ -885,10 +922,11 @@ def dictionaries(
     # type: (...) -> SearchStrategy[Dict[Ex, T]]
     # Describing the exact dict_class to Mypy drops the key and value types,
     # so we report Dict[K, V] instead of Mapping[Any, Any] for now.  Sorry!
-    """Generates dictionaries of type dict_class with keys drawn from the keys
-    argument and values drawn from the values argument.
+    """Generates dictionaries of type ``dict_class`` with keys drawn from the ``keys``
+    argument and values drawn from the ``values`` argument.
 
-    The size parameters have the same interpretation as for lists.
+    The size parameters have the same interpretation as for
+    :func:`~hypothesis.strategies.lists`.
 
     Examples from this strategy shrink by trying to remove keys from the
     generated dictionary, and by shrinking each generated key and value.
@@ -1024,7 +1062,7 @@ def text(
 ):
     # type: (...) -> SearchStrategy[Text]
     """Generates values of a unicode text type (unicode on python 2, str on
-    python 3) with values drawn from alphabet, which should be an iterable of
+    python 3) with values drawn from ``alphabet``, which should be an iterable of
     length one strings or a strategy generating such strings.
 
     The default alphabet strategy can generate the full unicode range but
@@ -1032,7 +1070,7 @@ def text(
     encoding.  You can use :func:`~hypothesis.strategies.characters` without
     arguments to find surrogate-related bugs such as :bpo:`34454`.
 
-    min_size and max_size have the usual interpretations.
+    ``min_size`` and ``max_size`` have the usual interpretations.
     Note that Python measures string length by counting codepoints: U+00C5
     ``Å`` is a single character, while U+0041 U+030A ``Å`` is two - the ``A``,
     and a combining ring above.
@@ -1051,28 +1089,24 @@ def text(
     elif isinstance(alphabet, SearchStrategy):
         char_strategy = alphabet
     else:
-        char_strategy = sampled_from(list(alphabet)) if alphabet else nothing()
-        if not isinstance(alphabet, abc.Sequence):
-            raise InvalidArgument(
-                "alphabet must be an ordered sequence, or tests may be "
-                "flaky and shrinking weaker, but a %r is not a type of "
-                "sequence." % (type(alphabet),)
-            )
         non_string = [c for c in alphabet if not isinstance(c, string_types)]
         if non_string:
             raise InvalidArgument(
                 "The following elements in alphabet are not unicode "
                 "strings:  %r" % (non_string,)
             )
-        not_one_char = [
-            c for c in alphabet if isinstance(c, string_types) and len(c) != 1
-        ]
+        not_one_char = [c for c in alphabet if len(c) != 1]
         if not_one_char:
             raise InvalidArgument(
                 "The following elements in alphabet are not of length "
                 "one, which leads to violation of size constraints:  %r"
                 % (not_one_char,)
             )
+        char_strategy = (
+            characters(whitelist_categories=(), whitelist_characters=alphabet)
+            if alphabet
+            else nothing()
+        )
     if (max_size == 0 or char_strategy.is_empty) and not min_size:
         return just(u"")
     return StringStrategy(lists(char_strategy, min_size=min_size, max_size=max_size))
@@ -1161,7 +1195,6 @@ class RandomSeeder(object):
 
 class RandomModule(SearchStrategy):
     def do_draw(self, data):
-        data.can_reproduce_example_from_repr = False
         seed = data.draw(integers(0, 2 ** 32 - 1))
         seed_all, restore_all = get_seeder_and_restorer(seed)
         seed_all()
@@ -1293,7 +1326,7 @@ def from_type(thing):
     naive time and datetime strategies.
 
     The resolution logic may be changed in a future version, but currently
-    tries these four options:
+    tries these five options:
 
     1. If ``thing`` is in the default lookup mapping or user-registered lookup,
        return the corresponding strategy.  The default lookup covers all types
@@ -1304,7 +1337,13 @@ def from_type(thing):
        the union of the strategies for those types that are not subtypes of
        other elements in the lookup.
     4. Finally, if ``thing`` has type annotations for all required arguments,
-       it is resolved via :func:`~hypothesis.strategies.builds`.
+       and is not an abstract class, it is resolved via
+       :func:`~hypothesis.strategies.builds`.
+    5. Because :mod:`abstract types <python:abc>` cannot be instantiated,
+       we treat abstract types as the union of their concrete subclasses.
+       Note that this lookup works via inheritance but not via
+       :obj:`~python:abc.ABCMeta.register`, so you may still need to use
+       :func:`~hypothesis.strategies.register_type_strategy`.
 
     There is a valuable recipe for leveraging ``from_type()`` to generate
     "everything except" values from a specified type. I.e.
@@ -1319,12 +1358,26 @@ def from_type(thing):
 
     For example, ``everything_except(int)`` returns a strategy that can
     generate anything that ``from_type()`` can ever generate, except for
-    instances of :class:python:int, and excluding instances of types
-    added via :func:~hypothesis.strategies.register_type_strategy.
+    instances of :class:`python:int`, and excluding instances of types
+    added via :func:`~hypothesis.strategies.register_type_strategy`.
 
     This is useful when writing tests which check that invalid input is
     rejected in a certain way.
     """
+    if (
+        hasattr(typing, "_TypedDictMeta")
+        and type(thing) is typing._TypedDictMeta  # type: ignore
+    ):  # pragma: no cover
+        # The __optional_keys__ attribute may or may not be present, but if there's no
+        # way to tell and we just have to assume that everything is required.
+        # See https://github.com/python/cpython/pull/17214 for details.
+        optional = getattr(thing, "__optional_keys__", ())
+        anns = {k: from_type(v) for k, v in thing.__annotations__.items()}
+        return fixed_dictionaries(  # type: ignore
+            mapping={k: v for k, v in anns.items() if k not in optional},
+            optional={k: v for k, v in anns.items() if k in optional},
+        )
+
     # TODO: We would like to move this to the top level, but pending some major
     # refactoring it's hard to do without creating circular imports.
     from hypothesis.searchstrategy import types
@@ -1361,9 +1414,7 @@ def from_type(thing):
                 return one_of([from_type(t) for t in args])
         # We can't resolve forward references, and under Python 3.5 (only)
         # a forward reference is an instance of type.  Hence, explicit check:
-        elif (
-            hasattr(typing, "_ForwardRef") and type(thing) == typing._ForwardRef
-        ):  # pragma: no cover
+        elif type(thing) == getattr(typing, "_ForwardRef", None):  # pragma: no cover
             raise ResolutionFailed(
                 "thing=%s cannot be resolved.  Upgrading to python>=3.6 may "
                 "fix this problem via improvements to the typing module." % (thing,)
@@ -1391,7 +1442,7 @@ def from_type(thing):
     # not included because bool is a subclass of int as well as Number.
     strategies = [
         as_strategy(v, thing)
-        for k, v in types._global_type_lookup.items()
+        for k, v in sorted(types._global_type_lookup.items(), key=repr)
         if isinstance(k, type)
         and issubclass(k, thing)
         and sum(types.try_issubclass(k, typ) for typ in types._global_type_lookup) == 1
@@ -1417,7 +1468,15 @@ def from_type(thing):
             "using register_type_strategy" % (thing,)
         )
     # Finally, try to build an instance by calling the type object
-    return builds(thing)
+    if not isabstract(thing):
+        return builds(thing)
+    subclasses = thing.__subclasses__()
+    if not subclasses:
+        raise ResolutionFailed(
+            "Could not resolve %r to a strategy, because it is an abstract type "
+            "without any subclasses. Consider using register_type_strategy" % (thing,)
+        )
+    return sampled_from(subclasses).flatmap(from_type)
 
 
 @cacheable
@@ -1430,13 +1489,13 @@ def fractions(
     # type: (...) -> SearchStrategy[Fraction]
     """Returns a strategy which generates Fractions.
 
-    If min_value is not None then all generated values are no less than
-    min_value.  If max_value is not None then all generated values are no
-    greater than max_value.  min_value and max_value may be anything accepted
+    If ``min_value`` is not None then all generated values are no less than
+    ``min_value``.  If ``max_value`` is not None then all generated values are no
+    greater than ``max_value``.  ``min_value`` and ``max_value`` may be anything accepted
     by the :class:`~fractions.Fraction` constructor.
 
-    If max_denominator is not None then the denominator of any generated
-    values is no greater than max_denominator. Note that max_denominator must
+    If ``max_denominator`` is not None then the denominator of any generated
+    values is no greater than ``max_denominator``. Note that ``max_denominator`` must
     be None or a positive integer.
 
     Examples from this strategy shrink towards smaller denominators, then
@@ -1444,14 +1503,9 @@ def fractions(
     """
     min_value = try_convert(Fraction, min_value, "min_value")
     max_value = try_convert(Fraction, max_value, "max_value")
-
-    if (
-        min_value is not None
-        and not isinstance(min_value, Fraction)
-        or max_value is not None
-        and not isinstance(max_value, Fraction)
-    ):
-        assert False, "Unreachable for Mypy"  # pragma: no cover
+    # These assertions tell Mypy what happened in try_convert
+    assert min_value is None or isinstance(min_value, Fraction)
+    assert max_value is None or isinstance(max_value, Fraction)
 
     check_valid_interval(min_value, max_value, "min_value", "max_value")
     check_valid_integer(max_denominator)
@@ -2058,6 +2112,7 @@ class RunnerStrategy(SearchStrategy):
 
 @defines_strategy_with_reusable_values
 def runner(default=not_set):
+    # type: (Any) -> SearchStrategy[Any]
     """A strategy for getting "the current test runner", whatever that may be.
     The exact meaning depends on the entry point, but it will usually be the
     associated 'self' value for it.
@@ -2087,6 +2142,7 @@ class DataObject(object):
 
     def draw(self, strategy, label=None):
         # type: (SearchStrategy[Ex], Any) -> Ex
+        check_type(SearchStrategy, strategy, "strategy")
         result = self.conjecture_data.draw(strategy)
         self.count += 1
         if label is not None:
@@ -2100,8 +2156,6 @@ class DataStrategy(SearchStrategy):
     supports_find = False
 
     def do_draw(self, data):
-        data.can_reproduce_example_from_repr = False
-
         if not hasattr(data, "hypothesis_shared_data_strategy"):
             data.hypothesis_shared_data_strategy = DataObject(data)
         return data.hypothesis_shared_data_strategy
@@ -2134,10 +2188,6 @@ def data():
     """This isn't really a normal strategy, but instead gives you an object
     which can be used to draw data interactively from other strategies.
 
-    It can only be used within :func:`@given <hypothesis.given>`, not
-    :func:`find() <hypothesis.find>`. This is because the lifetime
-    of the object cannot outlast the test body.
-
     See :ref:`the rest of the documentation <interactive-draw>` for more
     complete information.
 
@@ -2220,6 +2270,7 @@ def deferred(definition):
 
 @defines_strategy_with_reusable_values
 def emails():
+    # type: () -> SearchStrategy[Text]
     """A strategy for generating email addresses as unicode strings. The
     address format is specified in :rfc:`5322#section-3.4.1`. Values shrink
     towards shorter local-parts and host domains.
@@ -2237,14 +2288,14 @@ def emails():
     )
 
 
-# Mypy can't yet handle default values with generic types or typevars, but the
-# @overload workaround from https://github.com/python/mypy/issues/3737 doesn't
-# work with @composite functions - Mypy can't see that the function implements
-# `(Any, Callable[..., T], SearchStrategy[T]) -> Callable[..., T]`
-
-
 @defines_strategy
-def functions(like=lambda: None, returns=none()):
+def functions(
+    like=lambda: None,  # type: Callable[..., Any]
+    returns=none(),  # type: SearchStrategy[Any]
+):
+    # type: (...) -> SearchStrategy[Callable[..., Any]]
+    # The proper type signature of `functions()` would have T instead of Any, but mypy
+    # disallows default args for generics: https://github.com/python/mypy/issues/3737
     """A strategy for functions, which can be used in callbacks.
 
     The generated functions will mimic the interface of ``like``, which must
@@ -2269,6 +2320,7 @@ def functions(like=lambda: None, returns=none()):
 
 @composite
 def slices(draw, size):
+    # type: (Any, int) -> slice
     """Generates slices that will select indices up to the supplied size
 
     Generated slices will have start and stop indices that range from 0 to size - 1
